# Configure PuppetDB
class profile::puppet::server::puppetdb (
  Stdlib::Absolutepath $puppet_ssl_key_path = "${puppet::ssldir}/private_keys/puppet.pem",
  Stdlib::Absolutepath $puppet_ssl_cert_path = "${puppet::ssldir}/certs/puppet.pem",
  Stdlib::Absolutepath $puppet_ssl_ca_cert_path = "${puppet::ssldir}/certs/ca.pem",
  Stdlib::Absolutepath $puppetdb_local_dir = '/var/local/lib/puppetdb',
  Stdlib::Absolutepath $puppetdb_ssl_dir = "${puppetdb_local_dir}/ssl",
  Stdlib::Absolutepath $puppetdb_ssl_key_path = "${puppetdb_ssl_dir}/private.pem",
  Stdlib::Absolutepath $puppetdb_ssl_cert_path = "${puppetdb_ssl_dir}/public.pem",
  Stdlib::Absolutepath $puppetdb_ssl_ca_cert_path = "${puppetdb_ssl_dir}/ca.pem",
  String $puppetdb_user = 'puppetdb',
  String $puppetdb_group = 'puppetdb',
) {

  # The `puppetdb` user is not declared in the puppetdb module, so we declare
  # it here.
  user { 'puppetdb':
    ensure => present,
    system => true,
  }

  # Use Puppet's TLS certificates to configure Puppetdb. We use a directory in
  # /var/local to avoid too much ordering and creating circular dependencies.

  # TLS material is generated by Puppet Master the first time it runs, but we
  # also want to configure access to PuppetDB using those very same
  # certificates. To make that feasible while avoiding the creation of circular
  # dependencies between Puppet and Puppetdb, we put the TLS material somewhere
  # Puppetdb will be able to read once it's installed.
  exec { "Create ${puppetdb_local_dir}":
    creates => $puppetdb_local_dir,
    command => "mkdir -p ${puppetdb_local_dir}",
    path    => $::path,
  } -> file {
    $puppetdb_ssl_dir:
      ensure => directory,
      owner  => $puppetdb_user,
      group  => $puppetdb_group,
      mode   => '0700';
    $puppetdb_ssl_key_path:
      ensure  => file,
      source  => "file://${puppet_ssl_key_path}",
      owner   => $puppetdb_user,
      group   => $puppetdb_group,
      require => User['puppetdb'],
      notify  => Service['puppetdb'],
      mode    => '0600';
    $puppetdb_ssl_cert_path:
      ensure  => file,
      source  => "file://${puppet_ssl_cert_path}",
      owner   => $puppetdb_user,
      group   => $puppetdb_group,
      require => User['puppetdb'],
      notify  => Service['puppetdb'],
      mode    => '0600';
    $puppetdb_ssl_ca_cert_path:
      ensure  => file,
      source  => "file://${puppet_ssl_ca_cert_path}",
      owner   => $puppetdb_user,
      group   => $puppetdb_group,
      require => User['puppetdb'],
      notify  => Service['puppetdb'],
      mode    => '0600';
  } ~> Service['puppetdb']

  include puppet::server::service

  class { 'puppetdb':
    certificate_whitelist_file => '/etc/puppetdb/certificate-whitelist',
    confdir                    => '/etc/puppetdb/conf.d',
    database_username          => 'puppet_db',
    database_name              => 'puppet_db',
    puppetdb_user              => $puppetdb_user,
    puppetdb_group             => $puppetdb_group,
    manage_package_repo        => false,
    postgres_version           => '15',
    ssl_set_cert_paths         => true,
    ssl_dir                    => $puppetdb_ssl_dir,
    ssl_key_path               => $puppetdb_ssl_key_path,
    ssl_cert_path              => $puppetdb_ssl_cert_path,
    ssl_ca_cert_path           => $puppetdb_ssl_ca_cert_path,
    ssl_listen_address         => '127.0.0.1',
    vardir                     => '/var/lib/puppetdb',
  }

  class { 'puppetdb::master::config':
    puppetdb_server             => 'puppet',
    puppetdb_port               => 8081,
    puppetdb_soft_write_failure => false,
    manage_storeconfigs         => false,
    restart_puppet              => false,
    terminus_package            => 'puppet-terminus-puppetdb',
  }
  Class['puppetdb::master::puppetdb_conf'] ~> Class['puppet::server::service']

}
